@@ -2,6 +2,7 @@ import json
 
 from django.db import transaction
 from django.http.response import HttpResponseBadRequest, JsonResponse
+from django.utils.encoding import force_str
 from django.utils.functional import cached_property
 from django.views.generic.base import ContextMixin, TemplateResponseMixin, View
 from django.views.generic.detail import SingleObjectMixin
@@ -20,8 +21,7 @@ class IncompleteSelectResponseMixin:
     """
     def get(self, request, **kwargs):
         if request.accepts('application/json') and 'field' in request.GET:
-            if 'query' in request.GET or 'offset' in request.GET:
-                return self._fetch_options(request)
+            return self._fetch_options(request)
         return super().get(request, **kwargs)
 
     def _fetch_options(self, request):
@@ -31,26 +31,44 @@ class IncompleteSelectResponseMixin:
         except KeyError:
             return HttpResponseBadRequest(f"No such field: {field_path}")
         assert isinstance(field.widget, (Selectize, DualSelector))
+        widget = field.widget
+        queryset = widget.choices.queryset
+        data = {'total_count': queryset.count()}
+
         try:
             offset = int(request.GET.get('offset'))
         except TypeError:
             offset = 0
-        if query := request.GET.get('query'):
-            data = {'query': query}
-            queryset = field.widget.search(query)
+
+        if widget.filter_by and any(k.startswith('filter-') for k in request.GET.keys()):
+            filters = {key: request.GET.getlist(f'filter-{key}') for key in widget.filter_by.keys()}
+            data['filters'] = filters
+            queryset = queryset.filter(widget.build_filter_query(filters))
+
+        if search := request.GET.get('search'):
+            data['search'] = search
+            queryset = queryset.filter(widget.build_search_query(search))
             incomplete = None  # incomplete state unknown
         else:
-            data = {}
-            queryset = field.widget.choices.queryset
-            incomplete = queryset.count() - offset > field.widget.max_prefetch_choices
-        limited_qs = queryset[offset:offset + field.widget.max_prefetch_choices]
+            incomplete = queryset.count() - offset > widget.max_prefetch_choices
+
+        limited_qs = queryset[offset:offset + widget.max_prefetch_choices]
         to_field_name = field.to_field_name if field.to_field_name else 'pk'
-        items = [{'id': getattr(item, to_field_name), 'label': str(item)} for item in limited_qs]
+        if widget.group_field_name:
+            options = [{
+                'id': getattr(item, to_field_name),
+                'label': str(item),
+                'optgroup': force_str(getattr(item, widget.group_field_name)),
+            } for item in limited_qs]
+        else:
+            options = [{
+                'id': getattr(item, to_field_name),
+                'label': str(item),
+            } for item in limited_qs]
         data.update(
-            count=len(items),
-            total_count=field.widget.choices.queryset.count(),
+            count=len(options),
             incomplete=incomplete,
-            items=items,
+            options=options,
         )
         return JsonResponse(data)
 
